/*
 * $Header: /CustEd/AutoArray.h 7     11.12.01 20:41 Oslph312 $
 *
 * This class wraps a pointer to an array allocated on the heap,
 * deleting the array when the class goes out of scope.
 *
 * AutoString is a template specialization for LPTSTRs.
 *
 * HINT: To see the string wrapped by AutoArray during Visual C++
 * debugging, add the following line to the AUTOEXP.DAT file:
 * AutoArray<*>=_ptr=<_ptr>
 *
 * A typical location for this file is:
 * C:\Program Files\Microsoft Visual Studio\Common\MSDev98\Bin\AUTOEXP.DAT
 */

#pragma once


template< class T >
class AutoArray {
private:
   T *_ptr;

public:
   explicit AutoArray( int n ) throw() : _ptr( new T[ n ] ) {
      assert( 0 != _ptr );
   }
   explicit AutoArray( T *ptr = 0 ) throw() : _ptr( ptr ) {
   }
   ~AutoArray() {
      delete[] _ptr; // NOTE: It's OK to delete a null pointer.
      reset_pointer( _ptr );
   }
   void alloc( int n ) {
      assert( 0 == _ptr );
      _ptr = new T[ n ];
   }
   T ** operator &() {
      return &_ptr;
   }
   operator T *() throw() {
      return _ptr;
   }
   operator const T *() const throw() {
      return _ptr;
   }
   operator void *() throw() {
      return _ptr;
   }
   operator const void *() const throw() {
      return _ptr;
   }
};


typedef AutoArray< CHAR  > AutoStringA;
typedef AutoArray< WCHAR > AutoStringW;

#ifdef UNICODE
typedef AutoStringW AutoString;
#else
typedef AutoStringA AutoString;
#endif

// end of file
