/*
 * $Header: /Cleaner/GlobalSubclasser.cpp 6     22.03.02 12:52 Oslph312 $
 */

#include "precomp.h"
#include "GlobalSubclasser.h"
#include "Exception.h"
#include "addAtom.h"
#include "winUtils.h"
#include "utils.h"
#include "trace.h"


GlobalSubclasser::GlobalSubclasser( 
   LPCTSTR pszWndClass, WNDPROC wndProcNew ) 

   : m_wndProc( wndProcNew )
   , m_wndProcSaved( 0 )
   , m_pszWndClass( pszWndClass )
{
   assert( isGoodStringPtrOrAtom( pszWndClass ) );
   assert( isGoodCodePtr( wndProcNew ) );

   trace( _T( "Creating global subclassing %s\n" ), 
      stringFromAtom( pszWndClass ) );  

   HWND hwnd = CreateWindow( pszWndClass, _T( "" ), 
      WS_POPUP, 0, 0, 0, 0, HWND_DESKTOP, 0, getModuleHandle(), 0 );
   if ( IsWindow( hwnd ) ) {
      m_wndProcSaved = (WNDPROC) SetClassLong( 
         hwnd, GCL_WNDPROC, reinterpret_cast< LONG >( wndProcNew ) );
      verify( DestroyWindow( hwnd ) );
   } else {
      const DWORD dwErr = GetLastError();
      trace( _T( "Global subclassing failure %s\nReason: %s\n" ), 
         stringFromAtom( pszWndClass ),
         WinException( dwErr ).what() );  
   }
   if ( 0 == m_wndProcSaved ) {
      FatalAppExit( 0, _T( "Global subclassing failed" ) );
   }
}


GlobalSubclasser::~GlobalSubclasser() {

   trace( _T( "Destroying global subclasser %s\n" ), 
      stringFromAtom( m_pszWndClass ) );  

   try {
      assert( isGoodStringPtrOrAtom( m_pszWndClass ) );
      assert( 0 != m_wndProcSaved );
      if ( 0 != m_wndProcSaved ) {
         HWND hwnd = CreateWindow( m_pszWndClass, _T( "" ), WS_POPUP,
            0, 0, 0, 0, HWND_DESKTOP, 0, getModuleHandle(), 0 );
         if ( IsWindow( hwnd ) ) {
            assert( GetClassLong( hwnd, GCL_WNDPROC ) == 
               reinterpret_cast< DWORD >( m_wndProc ) );
            SetClassLong( hwnd, GCL_WNDPROC, 
               reinterpret_cast< LONG >( m_wndProcSaved ) );
            verify( DestroyWindow( hwnd ) );
         }
      }

#if 0 // Test exception handling using div by zero:
      int y = 0;
      int x = 1 / y;
#endif
   }
   // No exceptions may leave the destructor!
   // This one's always called at exit time anyway.
   catch ( ... ) {
      trace( _T( "*** Exception in Subclasser dtor ignored\n" ) );
   }
}

// end of file
